#                _____              ___ ____     _ _
#               |  ___| __ ___  ___|_ _|  _ \ __| | |__
#               | |_ | '__/ _ \/ _ \| || |_) / _` | '_ \
#               |  _|| | |  __/  __/| ||  __/ (_| | |_) |
#               |_|  |_|  \___|\___|___|_|   \__,_|_.__/
#
#	ipdb_lib.pl-$Revision: 1.20 $ $Date: 2002/03/28 05:24:50 $ <$Author: bapril $@freeipdb.org>
######################################################################


#--------------------------------------------------------------------------------------
# 	AddRA()
# Takes:
# 	Database Connection (from pg)
#	The name of the new RA
# Returns:
#	-1 on error
#	RA ID on sucess
#
sub AddRA(){
	my $conn = shift || &IPDBError(-1,"Did not supply DB connection");
	my $ra = shift || &IPDBError(-1,"Did not supply RA to lookup");
	my $str;
	#is there one by that name?
	unless(&LookupRA($conn,$ra)){
		$str = $conn->prepare("INSERT INTO RATABLE (NAME) VALUES ('$ra')");
		$str->execute;
		if($str->err){
			my $err = $DBI::errstr;
			&IPDBError(-1,"Could not lookup RA:  $err");
		}
		$str->finish;
		undef $str;
		#did we add it?
		if(!&LookupRA($conn,$ra)){
			&IPDBError(-1,"There has been a error adding this RA");
		} else {
			print "RA $ra Added\n";
			return(&LookupRA($conn,$ra));
		}
	} else {
		&IPDBError(-1,"RA allready exists.");
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	AddRegion()
# Takes:
#	Database Connection (from pg)
#	ID of the RA to use
#	Name of the new Region
#	IPv6 (If value block is IPV6)
# Returns:
#	-1 on error
#	Region ID on sucess
#
sub AddRegion(){
	my $conn = shift || &IPDBError(-1,"No DB connection supplied");
	my $ra = shift || &IPDBError(-1,"No RA supplied");
	my $region = shift || &IPDBError(-1,"No region supplied");
	my $v6 = shift;
	my $str;
	my $out;
	unless(&LookupRA($conn,$ra)){
		&IPDBError(-1,"RA not found in database");
	}
	if($region !~ /[A-Za-z0-9\-]/ ){ &IPDBError(-1,"No region defined, or invalid chars"); }
	if(&LookupRegion($conn,$region)){
		&IPDBError(-1,"The region exists in the database");
	} else {
		if($v6){
			$str = $conn->prepare("INSERT INTO REGIONTABLE (NAME,RA,V6) VALUES ('$region',$ra,'t')");	
		} else {
			$str = $conn->prepare("INSERT INTO REGIONTABLE (NAME,RA) VALUES ('$region',$ra)");	
		}
		$str->execute;
		if($str->err){
			my $err = $DBI::errstr;
			&IPDBError(-1,"Could not add Region:  $err");
		} else {
			$str->finish;
			undef $str;
			if($out = &LookupRegion($conn,$region)){
				print "REGION $region Added\n";
				return($out);
			} else {
				&IPDBError(-1,"Region not added");
			}
		}
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	LookupRegion()
# Takes:
#	Database Connection (from pg)
#	ID or name of region to find
# Returns:
#	-1 on error
#	0 on no record
#	ID or name of region on sucess (opposate of the input)	
#
sub LookupRegion(){
	my $conn = shift || &IPDBError(-1,"No DB connedtion supplied");
	my $region = shift || &IPDBError(-1,"No Region supplied");
	my @out;
	if($region =~ /^\d+$/){
		$str = $conn->prepare("SELECT NAME FROM REGIONTABLE WHERE ID = $region");
	} else {
		$str = $conn->prepare("SELECT ID FROM REGIONTABLE WHERE NAME = '$region'");
	}
	$num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not lookup Region:  $err");
	}
	$num =~ s/E.*//g;
	if($num){
		@out = $str->fetchrow;
		$str->finish;
		undef $str;
		return($out[0]);
	} else {
		$str->finish;
		undef $str;
		return(0);
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	LookupRA()
# Takes:
#       Database Connection (from pg)
#       ID or name of RA to find
# Returns:
#       -1 on error
#       0 on no RA
#       ID or name of RA on sucess (opposate of the input)
#
#
sub LookupRA(){
	my $conn = shift || &IPDBError(-1,"No database connection supplied");
	my $ra = shift || &IPDBError(-1,"No RA supplied");
	my $str;
	my @out;
	if($ra =~ /^\d+$/){
		$query = "SELECT NAME FROM RATABLE WHERE ID = $ra";
	} else {
		$query = "SELECT ID FROM RATABLE WHERE NAME = '$ra'";
	}
	$str = $conn->prepare($query);
	$num = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not lookup RA:  $err");
	}
	$num =~ s/E.*//g;
	if($num){
		@out = $str->fetchrow;
		$str->finish;
		undef $str;
		return($out[0]);
	} else {
		$str->finish;
		undef $str;
		return(0);
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	GetRAFromRegion()
# Takes:
#	DB connection
#	Region name or ID
# Returns:
#	-1 on error
#	RA ID
#
sub GetRAFromRegion(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $region = shift || &IPDBError(-1,"No Region Specified");
	my $str;
	if($region =~ /^\d+$/){
	} else {
		$region = &LookupRegion($conn,$region);
	}
	$str = $conn->prepare("SELECT RA FROM REGIONTABLE WHERE ID = $region");
	$num = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not lookup RA: $err"); 
	}
	$num =~ s/E.*//g;
	$num || &IPDBError(-1,"No result from database");
	my @out = $str->fetchrow;
	$str->finish;
	undef $str;
	return($out[0]);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	reclaim()
# Takes:
#	Database Connection (from pg)
#	ID of block to reclaim
# Returns:
#	-1 on error
#	0 on no block to reclaom
#	1 on sucess (no recursion nessary)
#	2 on sucess (recursion needed.)
#
sub reclaim(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $id = shift || &IPDBError(-1,"No Block ID supplied");
	my $childl;
	my $childr;
	my $parent;
	my $sibling;
	my $str = $conn->prepare("SELECT PARENT,CHILDL,CHILDR,REGION FROM IPDB WHERE ID = $id");
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not get parent/child info $err");
	}
	$num =~ s/E.*//g;
	if($num){
		@out = $str->fetchrow;
		$parent = $out[0];
		my $str = $conn->prepare("SELECT PARENT,CHILDL,CHILDR,REGION FROM IPDB WHERE ID = $id");
		$childl = $out[1];
		$childr = $out[2];
		$region = $out[3];
		my $num = $str->execute;
		if($num){
			@out = $str->fetchrow;
			$parentregion = $out[3];
		}
	} else {
		&IPDBError(-1,"Can't find block in database");
	}
	$str->finish;
	undef $str;
	if(($parent != 0 && !$childl && !$childr) && ($region == $parentregion)){ 
		&clearblock($conn,$id) ||&IPDBError(-1,"Could not clearblock");
		$sibling = &getsibling($conn,$id) || &IPDBError(-1,"Can't get sibling");
		if(&checksibling($conn,$sibling) == 1 && !&CheckReclaim($conn,$id)){
			&demoblock($conn,$id) || &IPDBError(-1,"Could not demoblock");
			&demoblock($conn,$sibling) || &IPDBError(-1,"Could not demoblock #2");
			&clearblock($conn,$parent) || &IPDBError(-1,"Error clearing block #1");
			my $oktoreclaim = &CheckReclaim($conn,$parent);
			if($oktoreclaim){
				return(0);
			} else {
				&reclaim($conn,$parent);
			}
			return(0);
		}
	} else {
		if($region != $parentregion){
			&clearblock($conn,$id)|| &IPDBError(-1,"Error clearing block #2");
			print "Did not traverse region boundry\n";
			return(0);
		}
		unless(!$childl || !$childr){
			&clearblock($conn,$id)|| &IPDBError(-1,"Error clearing block #3");
			return(0);
		} else {
			&IPDBError(-1,"Could not clearblock it has children");
		}
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	CheckReclaim()
# Takes:
#	Dabatase connection
#	A block ID
# Returns:
#	-1 on error
#	0 on OK to reclaim
#	1 on DO NOT RECLAIM
sub CheckReclaim(){
	my $conn = shift || &IPDBError(-1,"No Database connection") ;
	my $blockid = shift || &IPDBError(-1,"No block supplied");
	my $str = $conn->prepare("SELECT RECLAIM FROM IPDB WHERE ID = $blockid");
	my $num = $str->execute;
	if($str->err){ 
		my $err = $DBI::errstr;
		&IPDBError(-1,"There was an error quering the database: $err");
	}
	$num =~ s/E.*//g;
	if($num){
		my @out = $str->fetchrow;
		my $reclaim = $out[0];
		if($reclaim) {
			print "RECLAIM: $reclaim\n";
		} else {
			print "RECLAIM\n";
		}
		$str->finish;
		undef $str;
		return($reclaim);
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	ReadBlock()
# Takes:
#	Database Connection (from pg) to ipdb
#	A DB conn
#	ID of block to read
# Returns:
#	-1 on an error
#	0 on no block in the database
#	Block information string.
#
sub ReadBlock(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $block = shift || &IPDBError(-1,"No Block supplied");
	my $query = "SELECT i.BLOCK,i.BITS,r.name,i.ALLOCATED,i.CUSTNUM,i.CUSTDESC
		FROM IPDB i,REGIONTABLE r
		WHERE i.ID = $block
			AND i.REGION = r.ID";
	my $str = $conn->prepare($query);
	my $num = $str->execute();
	if($str->err){ 
		my $err = $DBI::errstr;
		&IPDBError(-1,"There was an error quering the database: $err");
	}
	$num =~ s/E.*//g;
	if($num){
		my @out = $str->fetchrow;
		my $dblock = $out[0];
		my $bits = $out[1];
		my $region = $out[2];
		my $allocated = $out[3];
		my $customer = $out[4];
		my $CUSTDESC = $out[5];
		my $city;
		my $v;
		$ipblock = &deci2ip($dblock,&Version($conn,$block)) || &IPDBError(-1,"Invalid Deci2IP");
		$v = &Version($conn,$block)|| &IPDBError(-1,"determine version");
		$string = "$ipblock/$bits from $region To: $customer IPv$v\n";
		$str->finish;
		undef $str;
		return($string);
	} else {
		&IPDBError(0,"Could not ReadBlock");
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	checksibling()
# Takes:
#	Database Connection (from pg) to ipdb
#	Sibling ID (Does the block with ths ID have any siblings)
# Returns:
#	1 for No children
#	0 for Children
#
sub checksibling(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $id = shift || &IPDBError(-1,"No Block id supplied");
	my $str = $conn->prepare("SELECT ID 
		FROM IPDB 
		WHERE CHILDL IS NULL 
			AND CHILDR IS NULL 
			AND ALLOCATED IS NULL 
			AND ID = $id");
	my $num = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"There was an error quering the database: $err");
        } 
	$str->finish;
	undef $str;
	my $oktoreclaim = &CheckReclaim($conn,$id);
	if($oktoreclaim){
		return(0);
	} else {
		$num =~ s/E.*//g;
		if($num){ return(1);} else {return(0);}
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	clearblock()
# Takes:
#	Database Connection (from pg) to ipdb
#	Block ID (The block to clear)
# Returns:
#	-1 for error
#	1 for sucess
#
sub clearblock(){
	my $conn = shift || &IPDBError(-1,"No Database Connection set");
	my $id = shift || &IPDBError(-1,"No blockID supplied");
	my $str = $conn->prepare("UPDATE IPDB SET CHILDR = NULL WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not set childr null on block $err");
	}
	$str = $conn->prepare("UPDATE IPDB SET CHILDL = NULL WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not set childl null on block $err");
	}
	$str = $conn->prepare("UPDATE IPDB SET CUSTDESC = NULL WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not set Customer Name null on block $err");
	}
	$str = $conn->prepare("UPDATE IPDB SET ALLOCATED = NULL WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not set allocated null on block $err");
	}
	$str = $conn->prepare("UPDATE IPDB SET CUSTNUM = NULL WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not set customer null on block $err");
	}
	my $now = time();
	my $hold = $now + $config{holdtime};
	$str = $conn->prepare("UPDATE IPDB SET HOLDTIME = $hold WHERE ID = $id");
	$str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not set holdtime on block $err");
	}
	$str->finish;
	undef $str;
	return(1);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	demoblock()
# Takes:
#	Database Connection (from pg) to ipdb
#	Block ID to demo.
# Returns:
#	-1 on error
#	1 on sucess
#
sub demoblock(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $id = shift || &IPDBError(-1,"No Block to destroy");
	my $str = $conn->prepare("DELETE FROM IPDB WHERE ID = $id");
	$str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		$str->finish;
		undef $str;
		&IPDBError(-1,"There was an error deleting the block $err");
	} else {
		$str->finish;
		undef $str;
		return(1);
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	getsibling()
# Takes:
#	Database Connection (from pg) to ipdb
#	Block ID to return sibling infor for.
# Returns:
#	-1 on error
#	ID of sibling on sucess.
#
sub getsibling(){
	my $conn = shift || &IPDBError(-1,"No Database COnnection supplied");
	my $id = shift || &IPDBError(-1,"No Block supplied");
	my $parent;
	my $childr;
	my $childl;
	my $str = $conn->prepare("SELECT PARENT FROM IPDB WHERE ID = $id");
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not get parent from database $err");
	}
	$num =~ s/E.*//g;
	if($num){
		@out = $str->fetchrow;
		$parent = $out[0];
	} else {
		&IPDBError(-1,"Child has no parent.");
	}
	$str->finish;
	undef $str;
	my $str2 = $conn->prepare("SELECT CHILDL,CHILDR FROM IPDB WHERE ID = $parent");
	my $num2 = $str2->execute;
	if($str2->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not get child info from database $err\n");
        }
	$num2 =~ s/E.*//g;
	if($num2){
		@out2 = $str2->fetchrow;
		$childl = $out2[0];
		$childr = $out2[1];
	} else {
		&IPDBError(-1,"No child information returned from database.");
	}
	$str2->finish;
	undef $str2;
	if($childl == $id){
		return($childr);
	}
	if($childr == $id){
		return($childl);
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	NewBlock()
# Takes:
#	Database Connection (from pg) to ipdb	
#	A region ID
#	The size of the new block (in bits)
#	A req #
#	A customer code.
# Returns:
#	-1 on error
#	0 on No Free Space
#	ID on sucess.
#
sub NewBlock(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $region = shift || &IPDBError(-1,"No region supplied");
	my $bits = shift || &IPDBError(-1,"No bits supplied");
	my $CUSTDESC = shift;
	my $CUST = shift ;
	my $id = "";
	my $newblock = "";
	my $now = time();
	my @out;
	# Any blocks the exact Size?
	my $str = $conn->prepare("SELECT ID,BITS
					FROM IPDB 
					WHERE CHILDL IS NULL 
						AND CHILDR IS NULL 
						AND ALLOCATED IS NULL  
						AND REGION = $region 	
						AND BITS <= $bits 
						AND RECLAIM IS NULL 
						AND (HOLDTIME < $now OR HOLDTIME IS NULL)
					ORDER BY priority ASC,BITS DESC");
	my $num = $str->execute;
	if($str->err){ 
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not pull free-blocks from the database $err");
	}
	while(@out = $str->fetchrow){
		my $size = $out[1];
		$id = $out[0];
		if($size == $bits){
			#We have exact blocks
			&setblock($conn,$id,$CUSTDESC,$CUST,$region) || &IPDBError(-1,"Unable to setblock");
			$str->finish;
			undef $str;
			return($id);
		} else {
			# There are larger blocks in region.
			$newblock = &makeblock($conn,$id,$bits) || &IPDBError(-1,"Unable to makeblock");
			&setblock($conn,$newblock,$CUSTDESC,$CUST,$region) || &IPDBError(-1,"Unable to setblock #2");
			$str->finish;
			undef $str;
			return($newblock);
		}
	}
	# No larger Blocks in region.
	print "No Free Blocks in region\n";
	return(0);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	ip2deci()
# Takes:
#	an IP-Address (IPv4 or IPv6)
# Returns:
#	A decimal number
#
sub ip2deci(){
	use Net::IP;
	my $in = shift || &IPDBError(-1,"No IP block supplied");
	my $ip =  new Net::IP ($in) or &IPDBError(-1,"IP address no supplied");
	$out = $ip->intip();
	return $out;
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#       setblock()
# Takes:
#       Database Connection (from pg) to ipdb
#       The block id of the block to set.
#       The Customer name to assign this allocation to.
#       Customer code/number
#       Optional Region
# Returns:
#       -1 on error or not in database.
#       1 on sucess
#
sub setblock(){
        my $conn = shift || &IPDBError(-1,"No Database connection");
        my $block = shift || &IPDBError(-1,"No Block ID");
        my $CUSTDESC = shift;
        my $CUST = shift;
        my $region = shift;
        my $time = time();
	#Shouldn't we get bits here too?
        my $str = $conn->prepare("SELECT ALLOCATED,CHILDL,CHILDR,RECLAIM FROM IPDB WHERE ID = $block");
	my $num = $str->execute;
        if($str->err){
		my $err = $DBI::errstr;
                &IPDBError(-1,"could not check database for block: $err");
        }
        my $allocated;
        my $childl;
        my $childr;
        my $reclaim;
	$num =~ s/E.*//g;
        if($num){
		@out = $str->fetchrow;
                $allocated = $out[0];
                $childl = $out[1];
                $childr = $out[2];
                $reclaim = $out[3];
		if ($allocated) {
			if($allocated =~ /\d+/){
				&IPDBError(-1,"Block in use  or allocated set to $allocated");
			}
		}
		if ($childl || $childr) {
			if($childl =~ /\d+/ || $childr =~ /\d+/){
				&IPDBError(-1,"Block $block has children");
			}
		}
        } else {
                &IPDBError(-1,"Block not in database");
        }
	if($reclaim){
        	if($reclaim =~ /[01]/){
			&IPDBError(0,"Unable to set block In reclaim mode");
        	}
	}
        $str = $conn->prepare("UPDATE IPDB SET ALLOCATED = $time WHERE ID = $block");
	$str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Unable to set Allocated time. $err");
	}
	$str = $conn->prepare("UPDATE IPDB SET HOLDTIME = NULL WHERE ID = $block");
        if($str->err){
		my $err = $DBI::errstr;
                &IPDBError(-1,"Unable to set allocated $err");
        }
	if($CUSTDESC){
        	$str = $conn->prepare("UPDATE IPDB SET CUSTDESC = '$CUSTDESC' WHERE ID = $block");
		$str->execute;
        	if($str->err){
			my $err = $DBI::errstr;
			&IPDBError(-1,"Unable to set  CUSTDESC($CUSTDESC) $block $err");
        	}
	}
        if($CUST){
                $str = $conn->prepare("UPDATE IPDB SET CUSTNUM = $CUST WHERE ID = $block");
		$str->execute;
                if($str->err){
			my $err = $DBI::errstr;
                        &IPDBError(-1,"unable to set customer code $err");
                }
        }
        if($region){
                $str = $conn->prepare("UPDATE IPDB SET REGION = $region WHERE ID = $block");
		$str->execute;
                if($str->err){
			my $err = $DBI::errstr;
                        &IPDBError(-1,"unable to set Region $err");
                }
        }
	$str->finish;
	undef $str;
        return(1);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	NewAlloc()
# Takes:
#	Database Connection (from pg) to ipdb
#	The block id of the block to set.
#	Region ID.
# Returns:
#	-1 on error or not in database.
#	ID on sucess
#
sub NewAlloc(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $block = shift || &IPDBError(-1,"No Block");
	my $bits = shift || &IPDBError(-1,"No Bits");
	my $region = shift || &IPDBError(-1,"No region Supplied");
	my $priority = shift;
	my $binblock = &ip2deci($block);
        my $check = &CheckBlockFree($conn,$binblock,$bits,$region);
        if($check){
                print "Block is in use<BR>\n";
                return(0);
        } else {
		$ver = &VersionFromRegion($conn,$region);
                my $added = &SetBlockP($conn,$block,$region,$bits,1,1,$ver);
                if($added){
                        print "A parent of that block exists.<BR>\n";
                        &reclaim($conn,$added);
                } else {
			unless($priority){$priority = "NULL";}
			my $str = $conn->prepare("INSERT INTO IPDB (BLOCK,BITS,REGION,PARENT,PRIORITY) VALUES ($binblock,$bits,$region,0,$priority)");
			$str->execute;
			if($str->err){
				my $err = $DBI::errstr;
				&IPDBError(-1,"Could not insert block into database| $err");
			}
			my $query = "SELECT ID FROM IPDB WHERE BLOCK = ".$binblock."::NUMERIC(40,0) AND BITS = $bits AND REGION = $region";
			$str = $conn->prepare($query);         
			my $num = $str->execute;
			if($str->err){
				my $err = $DBI::errstr;
				&IPDBError(-1,"Could not query block from database| $err");
			}
			$num =~ s/E.*//g;
			if($num){
				my @out = $str->fetchrow;
				$blockID = $out[0];
				print "<BR>Block added to database.";
				$str->finish;
				undef $str;
				return($blockID);
			} else {
				&IPDBError(-1,"Could not confirm block in database.");
                        }
                }
	}
}
#--------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------
#	splitblock()
# Takes:
#	Database Connection (from pg) to ipdb
#	ID of block to split
#	The decimal value of the block to be returned.
# Returns:
#	-1 on an error
#	0 on parent block not in database
#	ID of (red defined) child block in database
#
sub splitblock() {
	use Math::BigInt;
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $id = shift || &IPDBError(-1,"No Block ID supplied");
	my $ret = shift;
	my $str = $conn->prepare("SELECT BLOCK,BITS,REGION,RECLAIM,priority FROM IPDB WHERE ID = $id");
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Couldn't query block from database| $err");
	}
	$num =~ s/E.*//g;
	unless($num){
		&IPDBError(-1,"Parent block not in database ID = $id");
	}
	my @out = $str->fetchrow;
	my $block = $out[0];
	my $bits = $out[1];
	my $region = $out[2];
	my $reclaim = $out[3];
	my $alloc_prio = $out[4] || "NULL";
	my $allocated = $out[5];
	if($allocated){
		&IPDBError(-1,"Parent block is in use!");
	}
	if($reclaim =~ /[01]/){ &IPDBError(0,"Could not split block in-use");}
	$bits++;
	my $block1 = Math::BigInt->new($block);
	my $block2 = Math::BigInt->new($block);
	$block = Math::BigInt->new($block);
	my $temp = Math::BigInt->new("1");
	my $v;
	if(&Version($conn,$id) == 4){
		$temp = $temp->blsft((32 - $bits));
		$block2 = $block ^ $temp;
		$v = 4;
	} else {
		$temp = $temp->blsft((128 - $bits));
		$block2 = $block ^ $temp;
		$v = 6;
	}
	$block2 =~ s/\+//g; # Strip the leading "+"
	$block1 =~ s/\+//g;
	$str = $conn->prepare("INSERT INTO IPDB (BLOCK,BITS,REGION,PARENT,priority) VALUES ($block1,$bits,$region,$id,$alloc_prio)");
	$str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Error creating block1 Sent Block |$block1| BITS |$bits| REGION |$region| ID |$id| PRIO |$alloc_prio| $err");
	}
    $str = $conn->prepare("INSERT INTO IPDB (BLOCK,BITS,REGION,PARENT,priority) VALUES ($block2,$bits,$region,$id,$alloc_prio)");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Error creating block2 $err");
	}
	my $query1 = "SELECT ID FROM IPDB WHERE BLOCK = ".$block1."::numeric(40,0) AND BITS = $bits AND REGION = $region";
	my $query2 = "SELECT ID FROM IPDB WHERE BLOCK = ".$block2."::numeric(40,0) AND BITS = $bits AND REGION = $region";
    	my $str1 = $conn->prepare($query1);
	my $num1 = $str1->execute;
	if($str1->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Error quering for block1 $err");
	}
    	my $str2 = $conn->prepare($query2);
	my $num2 = $str2->execute;
	if($str2->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Error quering for block2 $err");
	}
	my $childr;
	my $childl;
	$num1 =~ s/E.*//g;
	if($num1){
			my @out1 = $str1->fetchrow;
        	$childl = $out1[0];
	} else {
		&IPDBError(-1,"There was an error creating block1");
	}
	$num2 =~ s/E.*//g;
	if($num2){
			my @out2 = $str2->fetchrow;
        	$childr = $out2[0];
	} else {
		&IPDBError(-1,"There was an error creating block2");
	}
	$str = $conn->prepare("UPDATE IPDB SET CHILDR = $childr WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Setting ChildR($childr) filed in parent($id) $err");
	}
	$str = $conn->prepare("UPDATE IPDB SET CHILDL = $childl WHERE ID = $id");
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Setting ChildL filed in parent $err");
	}
	my $a;
	$a = &makemask($bits,$v) || &IPDBError(0,"Makemask failed");
	my $mask = Math::BigInt->new($a);
	$block1 =~ s/^/+/g; # Put the leading "+" back on.
	$block2 =~ s/^/+/g;
	my $bm1 = $block1 & $mask;
	my $bm2 = $block2 & $mask;
	my $b1 = Math::BigInt->new($bm1);
	my $b2 = Math::BigInt->new($bm2);
	my $t1 = $ret & $mask;
	my $test = Math::BigInt->new($t1);
	$str->finish;
	undef $str;
	$str1->finish;
	undef $str1;
	$str2->finish;
	undef $str2;
	if(!$ret){
		if($test == "+0"){
			return($childl);
		} else {
			return($childr);
		}
	} else {
		if($block1 == $ret || $block1 ==  $t1){
			return($childl);
		} elsif ($block2 == $ret || $block2 == $t1){
			return($childr);
		} else {
			print "My output did not match the input\n";
		}
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	makemask()
# Takes:
#	Number of bits to mask
#	Version 4 || 6
# Returns:
#	-1 on error
#	a bitmask
#
sub makemask(){
	my $bits = shift || &IPDBError(-1,"No Mask-size supplied");
	my $version = shift || &IPDBError(-1,"No IP version specified");
	my $i;
	my $out = Math::BigInt->new("0");
	if($version == 4){ 
		$i = 31;
	} elsif($version == 6) {
		$i = 127;
	} else {
		&IPDBError(-1,"Did not supply an IP version");
	}
	if($bits < 1){
		&IPDBError(-1,"Did not supply mask size");
	}
	while($bits){
		my $temp = Math::BigInt->new("1");
		$temp = $temp->blsft($i);
		$out = $out ^ $temp;
		$bits--;
		$i--;
	}
	return($out);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	makeblock()
# Takes:
#	Database connection (pg)
#	Block to base from
#	Size in netmask.
# Returns:
#	-1 on error
#	0 on base block does not exist.
#	ID of new block
#
sub makeblock(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $id = shift || &IPDBError(-1,"No Block ID");
	my $bits = shift || &IPDBError(-1,"No Bits specified");
	my $str = $conn->prepare("SELECT BITS,REGION,BLOCK FROM IPDB WHERE ID = $id");
	my $num = $str->execute;
	if($str->err)	{
		my $err = $DBI::errstr;
		&IPDBError(-1,"There was an error iuering parent block $err");
	}
	$num =~ s/E.*//g;
	$num|| &IPDBError(-1,"Parent block does not exist.");
	my @out = $str->fetchrow;
	my $size = $out[0];
	my $region = $out[1];
	my $block = $out[2];
	my $foo = "";
	$foo = &splitblock($conn,$id,0) || &IPDBError(-1,"Splitblock failed");
	if(($size + 1) != $bits){ # the next block will fit
		$out1 = &makeblock($conn,$foo,$bits) || &IPDBError(-1,"Makeblock failed");
		$str->finish;
		undef $str;
		return($out1);
	}
	return($foo);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	deci2ip()
# Takes:
#	A Integet (to conver to an IP address)
#	A version (4 || 6);
# Returns:
#	The IP address in dotted decimal or IPv6 Notation.
#
sub deci2ip(){
        my ($binip,$ver) = @_;
	my $bin;
	my $ip;
	if($binip =~ m/[\d]+/){
		# Define normal size for address
		if($ver == 4){
			$bin = ip_inttobin($binip,4);
			$ip = ip_bintoip($bin,4);
			return($ip);
		} elsif($ver == 6) {	
			$bin = ip_inttobin($binip,6);
			$ip = ip_bintoip($bin,6);
			return($ip);
		} else {
			&IPDBError(-1,"Did not supply an IP version number");
		}	
	} else {
		&IPDBError(-1,"Did not get an integer to convert");
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	IPDB-Error()
# Takes:
#	Return Code (-1,1,0,...)
#	Text string
# Retruns
#	exits if RET = -1
#	0 if RET = 0
sub IPDBError(){
	my $code = shift;
	my $error = shift || "No Error supplied";
	my @sub = caller(1);
	my @me = caller();
	print  "Error in $sub[1] in $sub[3] at line $me[2] $error</TABLE>\n";
	warn "Error in $sub[1] in $sub[3] at line $me[2] $error\n";
	if($code < 0){ die("Exiting $code");}
	return($code);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	SetBlockP()
# Takes:
#	Database Connection (Pg);
#	Block ID to set.
#	Region to put block in.
#	Size of block in bits.
#	CUSTDESC Number 
#	CUST number
# Returns:
#	-1 on error
#	0 on block does not exist or is in use.
#	1 on sucess.
#
sub SetBlockP(){
        my $conn = shift || &IPDBError(-1,"DB connecton not supplied");
        my $block = shift || &IPDBError(-1,"Block ID not supplied");
        my $region = shift || &IPDBError(-1,"Region nit supplied");
        my $bits = shift || &IPDBError(-1,"Bits not supplied");
        my $CUSTDESC = shift;
	my $CUST = shift;
	my $ver = shift || &IPDBError(-1,"No IP version supplied 1");
	my $bblock = &ip2deci($block);
	my $newblock = &CheckBlockFree($conn,$bblock,$bits,$region);
	if($newblock == -1){
		print "That block is in use\n";
		$conn->disconnect;
		exit();
	}
	if($newblock == 0){
		$newblock = &MakeParent($conn,$bblock,$bits,$region,$bblock,$ver) || return(0);
	}
	&setblock($conn,$newblock,$CUSTDESC,$CUST,$region);
	return($newblock);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	CheckBlocksFree()
# Takes:
#	A database connection to IPDB
# 	A block ID to check for.
#	A A bit-boundry
# Returns:
#	-1 on Error
#	0 on No block found
#	ID on found block.
#
sub CheckBlockFree(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $block = shift || &IPDBError(-1,"No Block");
	my $bits = shift || &IPDBError(-1,"No Bits");
	my $region = shift || &IPDBError(-1,"No Region");
	$query = "SELECT ID FROM ipdb WHERE BITS = ".$bits." AND BLOCK = ".$block."::numeric(40) AND REGION = $region ";
	my $str = $conn->prepare($query);
	my $num = $str->execute;
	$num =~ s/E.*//g;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"There was an error with the database query $err"); 
	}
	unless($num){ 
		return(0);
	} else {
		my @out = $str->fetchrow;
		my $id = $out[0];
		my $str2 = $conn->prepare("SELECT ALLOCATED,CHILDR,CHILDL FROM IPDB WHERE ID = $id");
		$str2->execute;
		if($str2->err){
			my $err = $DBI::errstr;
			&IPDBError(-1,"There was an error with the database query $id $err");
		} else {
			my @out1 = $str2->fetchrow;
			my $allo = $out1[0];
			my $childl = $out1[1];
			my $childr = $out1[2];
			if($allo < 0 || $childl < 0 || $childr < 0){
				print "ID $id ALLO $allo CHILDR $childr CHILDL $childl\n";
				return(-1);
			} else {
				return($id);
			}
		}
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	ParentBlock()
# Takes:
#	block in integer form.
#	Bits 
#	version 4 or 6
# Returns:
#	Integer value of the parent
#
sub ParentBlock(){
	use Math::BigInt;
	my $block = shift || &IPDBError(-1,"No IP address specified");
	my $bits = shift || &IPDBError(-1,"No Netmask Specified");
	my $v = shift || &IPDBError(-1,"No IP Version Specified");
	my $BitFlip = Math::BigInt->new("1");
	my $other = Math::BigInt->new();
	my $out;
	$block = Math::BigInt->new($block);
	if($v == 6){
		$BitFlip = $BitFlip->blsft(128 - $bits);
		$other = ($block ^ $BitFlip);
	} elsif($v == 4){
		$BitFlip = $BitFlip->blsft(32 - $bits);
		$other = ($block ^ $BitFlip);
	} else {
		&IPDBError(-1,"IP Version not valid [$v]");
	}
	if($bits == 0){
                return(0);
        }
	if($block > $other){
		$out = $other;
	} else {
		$out = $block;
	}
	$out =~ s/\+//g;
	return($out);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	MakeParent()
# Takes:
#	Database connection
#	Block (integer format)
#	netmask in bits
#	region ID
#	Goal (The block we are trying to make)
#	Version (4||6)
# Returns:
#	ID of output block
#
sub MakeParent(){
	my $conn = shift || &IPDBError(-1,"No Database connection");
	my $block = shift || &IPDBError(-1,"No IP block");
	my $bits = shift || &IPDBErreo(-1,"No Netmask");
	my $region = shift;
	my $goal = shift || &IPDBError(-1,"No gloal set");
	my $ver = shift || &IPDBError(-1,"No Version");
	my $out;
	my $done;
	my $parent = &ParentBlock($block,$bits,$ver);
	if($parent == 0){return(0);} # Got to end of free blocks.
	if($bits == 1){&IPDBError(-1,"Block not on bit-boundry");}
	my $check = &CheckBlockFree($conn,$parent,($bits - 1),$region) ;
	if($check == -1 ){&IPDBError(-1,"Block come back in-use");}
	my $b1 = &deci2ip($block,$ver) || &IPDBError(-1,"Failed");
	my $b2 = &deci2ip($parent,$ver) || &IPDBError(-1,"Failed #2");
	if($check != 0){
		$done = &splitblock($conn,$check,$goal,$region) || &IPDBError(-1,"Splitblock failed");
	} else {
		$out = &MakeParent($conn,$parent,($bits -1),$region,$goal,$ver) || return(0) ;
		$done = &splitblock($conn,$out,$goal,$region) || &IPDBError(-1,"Splitblock failed #2");
	}
	return($done);
}	
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	ListRegions()
# Takes:
#	Database Connection (pg to ipdb)
# Returns:
#	prints a list (HTML Select list) of regions.
#
sub ListRegions(){
        my $conn = shift;
	my $str = $conn->prepare("SELECT ID,NAME FROM REGIONTABLE ORDER BY NAME");
	my @out;
	$str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	while (@out = $str->fetchrow) {
		print "<OPTION VALUE=$out[0]>$out[1]\n";
	}
	$str->finish;
	undef $str;
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	ListRA()
# Takes:
#	Database Connection (pg to ipdb)
# Returns:
#	prints a list (HTML Select list) of ra's
#
sub ListRA(){
	my $conn = shift;
	my $str = $conn->prepare("SELECT ID,NAME FROM RATABLE ORDER BY NAME");
	my @out;
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	while (@out = $str->fetchrow) {
		print "<OPTION VALUE=$out[0]>$out[1]\n";
	}
	$str->finish;
	undef $str;
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	Version()
# Takes:
#	Database Connection (pg to ipdb)
#	A block ID
# Returns:
#	(4||6) depending on version.
#
sub Version(){
	my $conn = shift;
	my $block = shift;
	my $query = "SELECT r.v6 from REGIONTABLE r,IPDB i WHERE i.REGION = r.ID AND i.ID = $block";
	my $str = $conn->prepare($query);
	$num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	$num =~ s/E.*//g;
	if($num){
			my @out = $str->fetchrow;
			my $v = $out[0];
			if($v){ return(6);} else { return(4);}
	} else {
			&IPDBError(0,"Block not in database");
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
sub VersionFromRegion(){
	my $conn = shift;
	my $region = shift;
	my $query = "SELECT v6 from REGIONTABLE WHERE ID = $region";
	my $str = $conn->prepare($query);
	$num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	$num =~ s/E.*//g;
	if($num){
		my @out = $str->fetchrow;
		my $v = $out[0];
		if($v){ return(6);} else { return(4);}
	} else {
		&IPDBError(0,"Region not in database.");
	}
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	GetBroadcat()
# Takes:
#	Block in int.
#	Size of block in bits.
# Returns:
#	Broadcast in int form.
#	
#
sub GetBroadcast(){
	use Math::BigInt;
	my $block = Math::BigInt->new(shift);
	my $size = shift;
	my $ver = shift;
	if($ver == 4){
		$num = 32;
	} elsif($ver == 6) {
		$num = 128;
	} else {
		&IPDBError(-1,"Incorrect Version supplied $ver");		
	}
	$size = $num - $size;
	while($size){
                $size--;
		my $temp = Math::BigInt->new("1");
		$temp = $temp->blsft($size);
		$block = $block | $temp;
        }
	$block =~ s/\+//g;
        return($block);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	Addresses()
# Takes:
# Returns:
#
sub Addresses(){
	my $size = shift;
	my $ver = shift;
	my $out = &GetBroadcast(0,$size,$ver);
	$out++; 
	return($out);
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	UtilDisplay()
# Takes:
#	A database connection to IPDB
# Prints:
#	A Block utilization report.
#
sub UtilDisplay(){
	my $conn = shift;
	my $query = "SELECT ID,BLOCK,BITS FROM IPDB WHERE PARENT = 0";
	my $str = $conn->prepare($query);
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	my $total = Math::BigInt->new("0");
	my $inuse = Math::BigInt->new("0");
	my $i = 0;
	while (@out = $str->fetchrow){
		$id = $out[0];
		$block = $out[1];
		$bits = $out[2];
		$ver = &Version($conn,$id);
		$total = &Addresses($bits,$ver);
		$blk = &deci2ip($block,$ver);
		$inuse = &GetInUse($conn,$id);	
		$pcnt = ($inuse/$total)*100;
		printf("%40s/%3d | %.2f %% in-use\n",$blk,$bits,$pcnt);
		undef $ver;
		undef $bits;
		undef $block;
		undef $id;
		undef $total;
		undef $inuse;
		undef $blk;
		undef $pcnt;
	}
	$str->finish;
	undef $str;
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	UtilRegion()
# Takes:
#	A database connection to FreeIPdb
# Returns: a list of region and % in use.
#
sub UtilRegion(){
	my $conn = shift;
	my $query = "SELECT BITS,ALLOCATED,REGION FROM IPDB WHERE CHILDL IS NULL ORDER BY REGION";
	my $str = $conn->prepare($query);
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	my $total = Math::BigInt->new("0");
	my $inuse = Math::BigInt->new("0");
	my $i = 0;
	my $number;
	my $region = "";
	my $oldregion = "";
	my $bits = "";
	while (@out = $str->fetchrow){
		$bits = $out[0];
		$alloc = $out[1];
		$region = $out[2];
		if($oldregion && ($oldregion ne $region)){
			$inuse =~ s/\+//g;
			$total =~ s/\+//g;
			$pcnt = ($inuse/$total)*100;
			$regionname = &LookupRegion($conn,$oldregion);
			printf("%40s | %.2f %% in-use\n",$regionname,$pcnt);
			$oldregion = $region;
			$total = Math::BigInt->new("0");
			$inuse = Math::BigInt->new("0");
		} else {
			$oldregion = $region;
		}
		$ver = &VersionFromRegion($conn,$region);
		$number = &Addresses($bits,$ver);
		$total += $number;
		if($alloc){ $inuse += $number;}
	}
	$inuse =~ s/\+//g;
	$total =~ s/\+//g;
	$pcnt = ($inuse/$total)*100;
        $regionname = &LookupRegion($conn,$oldregion);
	printf("%40s | %.2f %% in-use\n",$regionname,$pcnt);
	$str->finish;
	undef $str;
}
#--------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------
#	GetInUse()
# Takes:
#	A database connection to IPDB
#	The ID of a block to return in-use of.
# Returns:
#	A Math::BigInt od the numbre of addresses in use
#		recurssivly from this parent block
#
sub GetInUse(){
	my $conn = shift;
	my $block = shift;
	# Get info on children
	my $str = $conn->prepare("SELECT I.CHILDL,I.CHILDR,I.BITS,I.ALLOCATED,R.V6 FROM IPDB I, REGIONTABLE R WHERE I.ID = $block AND I.REGION = R.ID");
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	my @out = $str->fetchrow;
	my $childl = $out[0];
	my $childr = $out[1];
	my $size = $out[2];
	my $allocated = $out[3];
	my $ver = $out[4];
	if($ver){$ver = 6;} else { $ver = 4;}
	if(!$childl && !$childr){
		if($allocated){
			return(&Addresses($size,$ver));
		} else {
			return(0);
		}
	} 
	my $rtotal;
	my $ltotal;
	# Get info from childl
	my $strl = $conn->prepare("SELECT I.ALLOCATED,I.BITS,R.v6 FROM IPDB I,REGIONTABLE R WHERE I.ID = $childl AND I.REGION = R.ID");
	my $numl = $strl->execute;
	if($strl->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	$numl =~ s/E.*//g;
	if($numl){
		my @outl = $strl->fetchrow;
		if($outl[0]){
			my $verl = $outl[2];
			if($verl){$ver = 6;} else { $ver = 4;}
			$ltotal = &Addresses($outl[1],$ver);
		} else {
			$ltotal =  &GetInUse($conn,$childl);
		}
	} else {
		&IPDBError(-1,"Could not query Left block $childl.");
	}
	# Get info from childr
	my $strr = $conn->prepare("SELECT I.ALLOCATED,I.BITS,R.v6 FROM IPDB I,REGIONTABLE R WHERE I.ID = $childr AND I.REGION = R.ID");
	my $numr = $strr->execute;
	if($strr->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not query Database $err");
	}
	$numr =~ s/E.*//g;
	if($numr){
		my @outr = $strr->fetchrow;
		if($outr[0]){
			my $verr = $outr[2];
			if($verr){$ver = 6;} else { $ver = 4;}
			$rtotal = &Addresses($outr[1],$ver);
		} else {
			$rtotal = &GetInUse($conn,$childr);
		}
	} else {
		&IPDBError(-1,"Could not query Right Block.");
	}
	# Combine results
	my $out = $rtotal + $ltotal;
	# Give output
	$str->finish;
	undef $str;
	return($out);
}
#--------------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Subroutine ip_inttobin
# Purpose           : Transform a BigInt into a bit string
# Comments          : sets warnings (-w) off.
#                     This is necessary because Math::BigInt is not compliant
# Params            : BigInt, IP version
# Returns           : bit string
sub ip_inttobin
{
        require Math::BigInt;
        my $dec = Math::BigInt->new (shift);
        # Find IP version
        my $ip_version = shift;
        $ip_version or do
        {
                my $ERROR = "Cannot determine IP version for $dec";
                my $ERRNO = 101;
                return;
        };
        # Define normal size for address
	%IPLengths = ( 4 => 32 , 6 => 128);
        # Number of bits depends on IP version
        my $maxn = $IPLengths{$ip_version};
        my ($n, $binip);
        # Set warnings off, use integers only (loathe Math::BigInt)
        local $^W = 0;
        use integer;
        for ($n=0;$n < $maxn;$n++)
        {
                # Bit is 1 if $dec cannot be divided by 2
                $binip .= $dec%2;
                # Divide by 2, without fractional part
                $dec/= 2;
        };
        no integer;
        # Strip + signs
        $binip =~ s/\+//g;
        # Reverse bit string
        return scalar reverse $binip;
};

#------------------------------------------------------------------------------
# Subroutine ip_bintoip
# Purpose           : Transform a bit string into an IP address
# Params            : bit string, IP version
# Returns           : IP address on success, undef otherwise
sub ip_bintoip
{
        my ($binip,$ip_version) = @_;

	# Number of bits for each IP version
	%IPLengths = ( 4 => 32 , 6 => 128);

        # Define normal size for address
        my $len = $IPLengths{$ip_version};

        # Prepend 0s if address is less than normal size
        $binip = '0'x($len-length($binip)).$binip;

        # IPv4
        $ip_version == 4 and
                return join '.', unpack( 'C4C4C4C4', pack( 'B32', $binip ));

        # IPv6
        return join (':', unpack( 'H4H4H4H4H4H4H4H4', pack( 'B128', $binip )));
};

#------------------------------------------------------------------------------
# Subroutine ip_bintoint
# Purpose           : Transform a bit string into an Integer
# Params            : bit string
# Returns           : BigInt
sub ip_bintoint
{
        my $binip = shift;

        require Math::BigInt;

        # $n is the increment, $dec is the returned value
        my ($n,$dec) = (Math::BigInt->new (1),Math::BigInt->new (0));

        # Reverse the bit string
        foreach (reverse (split '', $binip))
        {
                # If the nth bit is 1, add 2**n to $dec
                $_ and $dec += $n;
                $n*=2;
        };
        # Strip leading + sign
        $dec=~s/^\+//;
        return $dec;
};

sub GetBlockId(){
        my $conn = shift;
        my $block = shift;
        my $bits = shift;
        my $region = shift;
        my $query = "SELECT ID FROM IPDB WHERE BLOCK = ".$block."::NUMERIC(40,0) AND BITS = $bits AND REGION = $region";
        my $str = $conn->prepare($query);
	my $num = $str->execute();
        if($str->err){ 
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not GetBlockId $err"); 
	}
        if($num){
		my @out = $str->fetchrow;
                my $id = $out[0];
                return($id);
        }   
	$str->finish;
	undef $str;
}

sub GetBlockFromID(){
	my $conn = shift;
	my $id = shift;
	my $query = "SELECT BLOCK,BITS,REGION,ALLOCATED,PARENT,CHILDL,CHILDR,CUSTDESC,CUSTNUM FROM IPDB WHERE ID = $id";
	my $str = $conn->prepare($query);
	my $num = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not GetBlockFromID $err"); 
	}
	$num =~ s/E.*//g;
	if($num){
		my @out = $str->fetchrow;
		return(@out);
	}
	$str->finish;
	undef $str;
}

# Returns 1 if block has any DNS associated with it.
sub CheckDNS(){
	my $conn = shift;
	my $id = shift;
	my $query = "SELECT ID FROM zone_record_table WHERE BLOCK = $id";
	$str = $conn->prepare($query);
	my $num = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not perform Check DNS query $err"); 
	}
	$str->finish;
	$num =~ s/E.*//g;
	if($num){
		return(1);
	} else {
		return(0);
	}
}

# Returns 1 if zone has any 
sub CheckZone(){
	my $conn = shift;
	my $id = shift;
	my $query = "SELECT ID FROM zone_record_table WHERE ZONE = $id";
	$str = $conn->prepare($query);
	my $num = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not perform CheckZone query $err"); 
	}
	$str->finish;
	$str = $conn->prepare("SELECT REVERSE_BLOCK FROM ZONE_TABLE WHERE ID = $id");
	my $num3 = $str->execute();
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not lookup reverse block $err"); 
	}
	@out = $str->fetchrow;
	if($out[0]){
		$num2 = &RevCheckZone($conn,$out[0]);
	}
	$num =~ s/E.*//g;
	$str->finish;
	if($num || $num2){
		return(1);
	} else {
		return(0);
	}
}

# Do the recursion for CheckZone
sub RevCheckZone(){
	my $conn = shift;
	my $block = shift || &IPDBError(-1,"Block not supplied.\n");
	my $str = $conn->prepare("SELECT CHILDL,CHILDR FROM IPDB WHERE ID = $block");
	my $num = $str->execute;
	if($str->err){
		my $err = $DBI::errstr;
		&IPDBError(-1,"Could not Find children blocks $err"); 
	}
	$num =~ s/E.*//g;
	if($num){
		my @out = $str->fetchrow;	
		my $childl = $out[0];
		my $childr = $out[1];	
		if($childr){
			my $return1 = &RevCheckZone($conn,$childl);
			my $return2 = &RevCheckZone($conn,$childr);
			$str->finish;
			return($return1 + $return2);
		} else {
			my $str2 = $conn->prepare("SELECT count(ID) FROM ZONE_RECORD_TABLE WHERE BLOCK = $block");
			my $num2 = $str2->execute;
			if($str2->err){
				my $err = $DBI::errstr;
				&IPDBError(-1,"Could not Find Check block for DNS $err"); 
			}
			@out = $str2->fetchrow;
			$str2->finish;
			return($out[0]);
		}
	}
}

1;
